<?php
/**
 * Copyright (©) 2022 Pinpoint Designs LTD. All right's reserved.
 * @category    Pinpoint
 * @copyright   Copyright (c) 2023 Pinpoint Designs. (http://www.pinpointdesigns.co.uk)
 * @author      Pavel Panchyshnyi <pavel.panchyshnyi@pinpointdesigns.co.uk>
 */

declare(strict_types=1);

namespace Pinpoint\ConversionTracking\ViewModel;

use Magento\Checkout\Model\Session;
use Magento\Directory\Model\CountryFactory;
use Magento\Framework\HTTP\Header;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\View\Element\Block\ArgumentInterface;
use Psr\Log\LoggerInterface;

class Conversion implements ArgumentInterface
{
    private Session $checkoutSession;
    private Json $serializer;
    private CountryFactory $countryFactory;
    private LoggerInterface $logger;
    private Header $httpHeader;

    /**
     * @param Session $checkoutSession
     * @param Json $serializer
     * @param CountryFactory $countryFactory
     * @param LoggerInterface $logger
     * @param Header $httpHeader
     */
    public function __construct(
        Session         $checkoutSession,
        Json            $serializer,
        CountryFactory  $countryFactory,
        LoggerInterface $logger,
        Header          $httpHeader
    ) {
        $this->checkoutSession = $checkoutSession;
        $this->serializer = $serializer;
        $this->countryFactory = $countryFactory;
        $this->logger = $logger;
        $this->httpHeader = $httpHeader;
    }

    /**
     * Returns serialized conversion data
     * @return boolean|string
     */
    public function getJson()
    {
        return $this->serializer->serialize($this->prepareConversionData());
    }

    /**
     * Prepares conversion data
     * @return array
     */
    private function prepareConversionData(): array
    {
        $order = $this->checkoutSession->getLastRealOrder();
        if (!$order->getId()) {
            $this->logger->debug('Order not found.', [
                'userAgent' => $this->httpHeader->getHttpUserAgent()
            ]);
            return [];
        }

        $address = $order->getIsVirtual()
            ? $order->getBillingAddress()
            : $order->getShippingAddress();

        $data = [
            'transaction-data' => [
                'date/time' => date('d/m/Y H:m'),
                'orderid' => $order->getIncrementId(),
                'useragent' => $this->httpHeader->getHttpUserAgent(),
                'email' => $order->getCustomerEmail(),
                'phone' => $address->getTelephone(),
                'firstname' => $address->getFirstname(),
                'lastname' => $address->getLastname(),
                'street' => $address->getStreetLine(1),
                'city' => $address->getCity(),
                'region' => $address->getRegion(),
                'country' => $this->getCountryName($address->getCountryId()),
                'postcode' => $address->getPostcode()
            ]
        ];

        $logData = array_merge(
            [
                $order->getIncrementId(),
                date('d/m/Y H:m'),
                $this->httpHeader->getHttpUserAgent()
            ], array_values($data['transaction-data'])
        );

        // Wrap all fields in " to imitate CSV fields
        array_walk($logData, function (&$row) {
            $row = '"' . $row . '"';
        });
        $logData =  implode(',', $logData);
        return $data;
    }

    /**
     * Retrieve entity country name
     * @param string $countryCode
     * @return string
     */
    private function getCountryName(string $countryCode): string
    {
        if ($countryCode) {
            $country = $this->countryFactory->create()->loadByCode($countryCode);
            return $country->getName();
        }
        return '';
    }
}
