<?php

declare(strict_types=1);

namespace Pinpoint\ConversionTracking\ViewModel;

use Magento\Catalog\Api\CategoryRepositoryInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\View\Element\Block\ArgumentInterface;
use Magento\Sales\Model\Order;
use Magento\Sales\Model\OrderFactory;
use Magento\Store\Model\ScopeInterface;

class SuccessPage implements ArgumentInterface
{
    const XML_PATH_CONVERSION_GENERAL = 'conversion/general/';
    /**
     * @var ScopeConfigInterface
     */
    private ScopeConfigInterface $scopeConfig;

    /**
     * @var OrderFactory
     */
    private OrderFactory $orderFactory;
    /**
     * @var \Magento\Catalog\Api\CategoryRepositoryInterface
     */
    private CategoryRepositoryInterface $categoryRepository;

    /**
     *  Success page constructor
     * @param ScopeConfigInterface $scopeConfig
     * @param OrderFactory $orderFactory
     * @param \Magento\Catalog\Api\CategoryRepositoryInterface $categoryRepository
     */
    public function __construct(
        ScopeConfigInterface        $scopeConfig,
        OrderFactory                $orderFactory,
        CategoryRepositoryInterface $categoryRepository
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->orderFactory = $orderFactory;
        $this->categoryRepository = $categoryRepository;
    }

    /**
     * Get conversion ID
     * @return string
     */
    public function getConversionId(): string
    {
        return $this->scopeConfig->getValue(self::XML_PATH_CONVERSION_GENERAL . "conversion_id", ScopeInterface::SCOPE_STORE);
    }

    /**
     * Get conversion label
     * @return string
     */
    public function getConversionLabel(): string
    {
        return $this->scopeConfig->getValue(self::XML_PATH_CONVERSION_GENERAL . "conversion_label", ScopeInterface::SCOPE_STORE);
    }

    /**
     * @param $orderId
     * @return Order
     */
    public function getOrder($orderId): Order
    {
        return $this->orderFactory->create()->loadByIncrementId($orderId);
    }

    /**
     * Get top level category name
     * @param $product
     * @return string|null
     */
    public function getCategoryName($product): ?string
    {
        $ids = array_diff($product->getCategoryIds(), ['2']);
        if (!empty($ids)) {
            sort($ids);
            $ids = array_reverse($ids);
            $catId = array_pop($ids);
            try {
                return $this->categoryRepository->get($catId)->getName();
            } catch (\Exception $e) {
            }
        }
        return "";
    }

    /**
     * Get brand name for product
     * @param $product
     * @return string|null
     */
    public function getBrand($product): ?string
    {
        $product->getAttributeText('manufacturer');
        $product->getAttributeText('brand');
        try {
            $brand = $product->getData('brand');
            if (!empty($brand)) {
                return $brand;
            }
            $manufacturer = $product->getData('manufacturer');
            if (!empty($manufacturer)) {
                return $manufacturer;
            }
        } catch (\Exception $e) {
        }
        return '';
    }

    /**
     * Get enabled data
     * @return array
     */
    public function getEnabledArray(): array
    {
        $enabled = [];
        foreach (["enable_gtag", "enable_datalayer", "enable_uetq"] as $config) {
            $enabled[$config] = (bool)$this->scopeConfig->getValue(self::XML_PATH_CONVERSION_GENERAL . $config, ScopeInterface::SCOPE_STORE);
        }
        return $enabled;
    }
}
